@@ -26,7 +26,9 @@ def get_arguments_from_annotations(
     for name, annotation in annotations.items():
         default_value = parameters[name].default
         default_value = (
-            undefined if default_value is inspect.Parameter.empty else default_value
+            undefined
+            if default_value is inspect.Parameter.empty or is_unset(default_value)
+            else default_value
         )
 
         argument_definition = ArgumentDefinition(
